@@ -41,6 +41,7 @@ namespace Theme {
 
 	fs::path theme_dir;
 	fs::path user_theme_dir;
+	fs::path custom_theme_dir;
 	vector<string> themes;
 	std::unordered_map<string, string> colors;
 	std::unordered_map<string, array<int, 3>> rgbs;
@@ -88,7 +89,8 @@ namespace Theme {
 		{ "upload_end", "#dcafde" },
 		{ "process_start", "#80d0a3" },
 		{ "process_mid", "#dcd179" },
-		{ "process_end", "#d45454" }
+		{ "process_end", "#d45454" },
+		{ "proc_pause_bg", "#b54040" }
 	};
 
 	const std::unordered_map<string, string> TTY_theme = {
@@ -133,7 +135,8 @@ namespace Theme {
 		{ "upload_end", "\x1b[95m" },
 		{ "process_start", "\x1b[32m" },
 		{ "process_mid", "\x1b[33m" },
-		{ "process_end", "\x1b[31m" }
+		{ "process_end", "\x1b[31m" },
+		{ "proc_pause_bg", "\x1b[41m" },
 	};
 
 	namespace {
@@ -304,7 +307,7 @@ namespace Theme {
 
 			for (const auto& [name, source_arr] : rgbs) {
 				if (not name.ends_with("_start")) continue;
-				const string color_name = rtrim(name, "_start");
+				const string color_name { rtrim(name, "_start") };
 
 				//? input_colors[start,mid,end][red,green,blue]
 				const array<array<int, 3>, 3> input_colors = {
@@ -357,7 +360,7 @@ namespace Theme {
 
 			for (const auto& c : colors) {
 				if (not c.first.ends_with("_start")) continue;
-				const string base_name = rtrim(c.first, "_start");
+				const string base_name { rtrim(c.first, "_start") };
 				string section = "_start";
 				int split = colors.at(base_name + "_mid").empty() ? 50 : 33;
 				for (int i : iota(0, 101)) {
@@ -372,13 +375,13 @@ namespace Theme {
 
 		//* Load a .theme file from disk
 		auto loadFile(const string& filename) {
-			std::unordered_map<string, string> theme_out;
 			const fs::path filepath = filename;
 			if (not fs::exists(filepath))
 				return Default_theme;
 
 			std::ifstream themefile(filepath);
 			if (themefile.good()) {
+				std::unordered_map<string, string> theme_out;
 				Logger::debug("Loading theme file: " + filename);
 				while (not themefile.bad()) {
 					if (themefile.peek() == '#') {
@@ -416,7 +419,8 @@ namespace Theme {
 		themes.push_back("Default");
 		themes.push_back("TTY");
 
-		for (const auto& path : { user_theme_dir, theme_dir } ) {
+		//? Priority: custom_theme_dir -> user_theme_dir -> theme_dir
+		for (const auto& path : { custom_theme_dir, user_theme_dir, theme_dir } ) {
 			if (path.empty()) continue;
 			for (auto& file : fs::directory_iterator(path)) {
 				if (file.path().extension() == ".theme" and access(file.path().c_str(), R_OK) != -1 and not v_contains(themes, file.path().c_str())) {
