@@ -62,6 +62,7 @@ tab-size = 4
 #include <regex>
 #include <string>
 #include <memory>
+#include <unordered_set>
 
 #include "../btop_config.hpp"
 #include "../btop_shared.hpp"
@@ -214,8 +215,7 @@ namespace Cpu {
 		return trim_name(string(buffer));
 	}
 
-	int64_t get_sensor(string device, sensor_type type, int num) {
-		int64_t temp = -1;
+	int64_t get_sensor(string device, vector<sensor_type> types, int num) {
 		struct sensordev sensordev;
 		struct sensor sensor;
 		size_t sdlen, slen;
@@ -233,25 +233,26 @@ namespace Cpu {
 					break;
 			}
 			if (strstr(sensordev.xname, device.c_str())) {
-				mib[3] = type;
 				mib[4] = num;
-				if (sysctl(mib, 5, &sensor, &slen, NULL, 0) == -1) {
-					if (errno != ENOENT) {
-						Logger::warning("sysctl");
-						continue;
-					}
+				for (sensor_type type : types) {
+					mib[3] = type;
+					if (sysctl(mib, 5, &sensor, &slen, NULL, 0) == -1) {
+						if (errno != ENOENT) {
+							Logger::warning("sysctl");
+							continue;
+						}
+					} else
+						return sensor.value;
 				}
-				temp = sensor.value;
-			 	break;
 			}
 		}
-		return temp;
+		return -1;
 	}
 
 	bool get_sensors() {
 		got_sensors = false;
 		if (Config::getB("show_coretemp") and Config::getB("check_temp")) {
-			if (get_sensor(string("cpu0") , SENSOR_TEMP, 0) > 0) {
+			if (get_sensor(string("cpu0"), { SENSOR_TEMP }, 0) > 0) {
 				got_sensors = true;
 				current_cpu.temp_max = 100; // we don't have this info
 			} else {
@@ -267,7 +268,7 @@ namespace Cpu {
 		int temp = 0;
 		int p_temp = 0;
 
-		temp = get_sensor(string("cpu0"), SENSOR_TEMP, 0);
+		temp = get_sensor(string("cpu0"), { SENSOR_TEMP }, 0);
 		if (temp > -1) {
 			temp = MUKTOC(temp);
 			p_temp = temp;
@@ -343,17 +344,23 @@ namespace Cpu {
 
 		long seconds = -1;
 		uint32_t percent = -1;
+		float rate = -1.0f;
 		string status = "discharging";
-		int64_t full, remaining;
-		full = get_sensor("acpibat0", SENSOR_AMPHOUR, 0);
-		remaining = get_sensor("acpibat0", SENSOR_AMPHOUR, 3);
-		int64_t state = get_sensor("acpibat0", SENSOR_INTEGER, 0);
+		int64_t full, remaining, watts;
+		full = get_sensor("acpibat0", { SENSOR_AMPHOUR, SENSOR_WATTHOUR }, 0);
+		remaining = get_sensor("acpibat0", { SENSOR_AMPHOUR, SENSOR_WATTHOUR }, 3);
+		watts = get_sensor("acpibat0", { SENSOR_WATTS }, 0);
+		int64_t state = get_sensor("acpibat0", { SENSOR_INTEGER }, 0);
 		if (full < 0) {
 			has_battery = false;
 			Logger::warning("failed to get battery");
 		} else {
 			float_t f = full / 1000;
 			float_t r = remaining / 1000;
+			if (watts > 0) {
+				supports_watts = true;
+				rate = watts / 1000000.0f; // watts is given in µW
+			}
 			has_battery = true;
 			percent = r / f * 100;
 			if (percent == 100) {
@@ -370,7 +377,7 @@ namespace Cpu {
 			}
 		}
 
-		return {percent, -1, seconds, status};
+		return {percent, rate, seconds, status};
 	}
 
 	auto collect(bool no_update) -> cpu_info & {
@@ -950,16 +957,18 @@ namespace Proc {
 	string current_sort;
 	string current_filter;
 	bool current_rev = false;
+	bool is_tree_mode;
 
 	fs::file_time_type passwd_time;
 
 	uint64_t cputimes;
-	int collapse = -1, expand = -1;
+	int collapse = -1, expand = -1, toggle_children = -1;
 	uint64_t old_cputimes = 0;
 	atomic<int> numpids = 0;
 	int filter_found = 0;
 
 	detail_container detailed;
+	static std::unordered_set<size_t> dead_procs;
 
 	string get_status(char s) {
 		if (s & SRUN) return "Running";
@@ -990,7 +999,9 @@ namespace Proc {
 		//? Process runtime : current time - start time (both in unix time - seconds since epoch)
 		struct timeval currentTime;
 		gettimeofday(&currentTime, nullptr);
-		detailed.elapsed = sec_to_dhms(currentTime.tv_sec - detailed.entry.cpu_s); // only interested in second granularity, so ignoring tc_usec
+		// only interested in second granularity, so ignoring tc_usec
+		if (detailed.entry.state != 'X') detailed.elapsed = sec_to_dhms(currentTime.tv_sec - detailed.entry.cpu_s);
+		else detailed.elapsed = sec_to_dhms(detailed.entry.death_time);
 		if (detailed.elapsed.size() > 8) detailed.elapsed.resize(detailed.elapsed.size() - 3);
 
 		//? Get parent process name
@@ -1021,14 +1032,17 @@ namespace Proc {
 		auto per_core = Config::getB("proc_per_core");
 		auto tree = Config::getB("proc_tree");
 		auto show_detailed = Config::getB("show_detailed");
+		const auto pause_proc_list = Config::getB("pause_proc_list");
 		const size_t detailed_pid = Config::getI("detailed_pid");
 		bool should_filter = current_filter != filter;
 		if (should_filter) current_filter = filter;
 		bool sorted_change = (sorting != current_sort or reverse != current_rev or should_filter);
+		bool tree_mode_change = tree != is_tree_mode;
 		if (sorted_change) {
 			current_sort = sorting;
 			current_rev = reverse;
 		}
+		if (tree_mode_change) is_tree_mode = tree;
 
 		const int cmult = (per_core) ? Shared::coreCount : 1;
 		bool got_detailed = false;
@@ -1061,11 +1075,16 @@ namespace Proc {
 				//? Check if pid already exists in current_procs
 				bool no_cache = false;
 				auto find_old = rng::find(current_procs, pid, &proc_info::pid);
+				//? Only add new processes if not paused
 				if (find_old == current_procs.end()) {
-					current_procs.push_back({pid});
-					find_old = current_procs.end() - 1;
-					no_cache = true;
+					if (not pause_proc_list) {
+						current_procs.push_back({pid});
+						find_old = current_procs.end() - 1;
+						no_cache = true;
+					}
+					else continue;
 				}
+				else if (dead_procs.contains(pid)) continue;
 
 				auto &new_proc = *find_old;
 
@@ -1118,9 +1137,31 @@ namespace Proc {
 				}
 			}
 
-			//? Clear dead processes from current_procs
-			auto eraser = rng::remove_if(current_procs, [&](const auto &element) { return not v_contains(found, element.pid); });
-			current_procs.erase(eraser.begin(), eraser.end());
+			//? Clear dead processes from current_procs if not paused
+			if (not pause_proc_list) {
+				auto eraser = rng::remove_if(current_procs, [&](const auto& element) { return not v_contains(found, element.pid); });
+				current_procs.erase(eraser.begin(), eraser.end());
+				if (!dead_procs.empty()) dead_procs.clear();
+			}
+			//? Set correct state of dead processes if paused
+			else {
+				for (auto& r : current_procs) {
+					if (rng::find(found, r.pid) == found.end()) {
+						if (r.state != 'X') {
+							struct timeval currentTime;
+							gettimeofday(&currentTime, nullptr);
+							r.death_time = currentTime.tv_sec - r.cpu_s;
+						}
+						r.state = 'X';
+						dead_procs.emplace(r.pid);
+						//? Reset cpu usage for dead processes if paused and option is set
+						if (!Config::getB("keep_dead_proc_usage")) {
+							r.cpu_p = 0.0;
+							r.mem = 0;
+						}
+					}
+				}
+			}
 
 			//? Update the details info box for process if active
 			if (show_detailed and got_detailed) {
@@ -1154,13 +1195,30 @@ namespace Proc {
 		}
 
 		//* Sort processes
-		if (sorted_change or not no_update) {
+		if ((sorted_change or tree_mode_change) or (not no_update and not pause_proc_list)) {
 			proc_sorter(current_procs, sorting, reverse, tree);
 		}
 
 		//* Generate tree view if enabled
 		if (tree and (not no_update or should_filter or sorted_change)) {
 			bool locate_selection = false;
+
+			if (toggle_children != -1) {
+				auto collapser = rng::find(current_procs, toggle_children, &proc_info::pid);
+				if (collapser != current_procs.end()){
+					for (auto& p : current_procs) {
+						if (p.ppid == collapser->pid) {
+							auto child = rng::find(current_procs, p.pid, &proc_info::pid);
+							if (child != current_procs.end()){
+								child->collapsed = not child->collapsed;
+							}
+						}
+					}
+					if (Config::ints.at("proc_selected") > 0) locate_selection = true;
+				}
+				toggle_children = -1;
+			}
+			
 			if (auto find_pid = (collapse != -1 ? collapse : expand); find_pid != -1) {
 				auto collapser = rng::find(current_procs, find_pid, &proc_info::pid);
 				if (collapser != current_procs.end()) {
@@ -1182,8 +1240,10 @@ namespace Proc {
 			vector<tree_proc> tree_procs;
 			tree_procs.reserve(current_procs.size());
 
-			for (auto& p : current_procs) {
-				if (not v_contains(found, p.ppid)) p.ppid = 0;
+			if (!pause_proc_list) {
+				for (auto& p : current_procs) {
+					if (not v_contains(found, p.ppid)) p.ppid = 0;
+				}
 			}
 
 			//? Stable sort to retain selected sorting among processes with the same parent
@@ -1196,7 +1256,7 @@ namespace Proc {
 
 			//? Recursive sort over tree structure to account for collapsed processes in the tree
 			int index = 0;
-			tree_sort(tree_procs, sorting, reverse, index, current_procs.size());
+			tree_sort(tree_procs, sorting, reverse, (pause_proc_list and not (sorted_change or tree_mode_change)), index, current_procs.size());
 
 			//? Recursive construction of ASCII tree prefixes.
 			for (auto t = tree_procs.begin(); t != tree_procs.end(); ++t) {
@@ -1204,7 +1264,7 @@ namespace Proc {
 			}
 
 			//? Final sort based on tree index
-			rng::sort(current_procs, rng::less{}, & proc_info::tree_index);
+			rng::stable_sort(current_procs, rng::less {}, &proc_info::tree_index);
 
 			//? Move current selection/view to the selected process when collapsing/expanding in the tree
 			if (locate_selection) {
