@@ -4,7 +4,7 @@
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at
 
-       http://www.apache.org/licenses/LICENSE-2.0
+	   http://www.apache.org/licenses/LICENSE-2.0
 
    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
@@ -16,9 +16,12 @@ indent = tab
 tab-size = 4
 */
 
+#include <Availability.h>
+#if __MAC_OS_X_VERSION_MIN_REQUIRED > 101504
 namespace Cpu {
 	class ThermalSensors {
 	   public:
 		long long getSensors();
 	};
 }  // namespace Cpu
+#endif
